@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "09/07/2021"
+__date__ = "26/10/2021"
 
 import copy
 import glob
@@ -42,6 +42,7 @@ import h5py
 import numpy
 
 import fabio
+import silx
 from silx.io import utils
 from silx.io import fabioh5
 from silx.io.url import DataUrl
@@ -105,7 +106,7 @@ class Dataset():
     """
 
     def __init__(self, _dir, data=None, first_filename=None, filenames=None,
-                 dims=None, transformation=None, in_memory=True, copy_files=False):
+                 dims=None, transformation=None, in_memory=True, copy_files=False, isH5=False):
 
         self._data = None
         self._frames_intensity = []
@@ -116,7 +117,7 @@ class Dataset():
         self._dir = _dir
         self._transformation = transformation
         if copy_files:
-            self._dir += "/treated"
+            self._dir = os.path.join(self._dir, "treated")
             if not os.path.isdir(self._dir):
                 try:
                     os.mkdir(self._dir)
@@ -131,27 +132,33 @@ class Dataset():
         # Keys: dimensions names, values: dimensions values
         self._dimensions_values = {}
         self._in_memory = in_memory
+        self._isH5 = isH5
 
         if data is not None:
             self._data = data
         else:
             if filenames is None and first_filename is None:
-                filenames = sorted([x for x in glob.glob(_dir + "/*") if os.path.isfile(x)])
+                filenames = sorted([x for x in glob.glob(os.path.join(_dir, "*")) if os.path.isfile(x)])
             self.filenames = filenames
             self.first_filename = first_filename
 
             metadata = []
             data_urls = []
 
-            with fabio.open_series(filenames=filenames, first_filename=first_filename) as series:
-                for frame in series.frames():
-                    filename = frame.file_container.filename
-                    data_urls.append(DataUrl(file_path=filename,
-                                             scheme='fabio'))
-                    fabio_reader = fabioh5.EdfFabioReader(file_name=filename)
-                    metadata.append(fabio_reader)
-                    fabio_reader.close()
-
+            if self._isH5:
+                url = DataUrl(first_filename)
+                with silx.io.open(url.file_path()) as h5:
+                    for i in range(h5[url.data_path()].shape[0]):
+                        data_urls.append(DataUrl(file_path=url.file_path(), data_path=url.data_path(), data_slice=i, scheme='silx'))
+            else:
+                with fabio.open_series(filenames=filenames, first_filename=first_filename) as series:
+                    for frame in series.frames():
+                        filename = frame.file_container.filename
+                        data_urls.append(DataUrl(file_path=filename,
+                                                 scheme='fabio'))
+                        fabio_reader = fabioh5.EdfFabioReader(file_name=filename)
+                        metadata.append(fabio_reader)
+                        fabio_reader.close()
             self._data = Data(numpy.array(data_urls), metadata=metadata, in_memory=self._in_memory)
 
     def stop_operation(self, operation):
@@ -182,10 +189,6 @@ class Dataset():
     def dir(self):
         return self._dir
 
-    @property
-    def treated(self):
-        return self._treated
-
     def compute_frames_intensity(self, kernel=(3, 3), sigma=20):
         """
         Returns for every image a number representing its intensity. This number
@@ -370,7 +373,7 @@ class Dataset():
                 return Dataset(_dir=self.dir, data=data, dims=self.__dims, in_memory=self._in_memory)
             except Exception:
                 raise ValueError("Failed to reshape data into dimensions {} \
-                                  Try using another tolerance value.".format(' '.join(self.__dims.get_names())))
+                                  Try using another tolerance or size values.".format(' '.join(self.__dims.get_names())))
         else:
             raise ValueError("Not enough dimensions where found")
 
@@ -390,8 +393,7 @@ class Dataset():
         self._dimensions_values = {}
 
         keys = numpy.array(list(self.data.metadata[0].get_keys(kind)))
-        values = numpy.array([[data.get_value(kind=kind, name=key)[0] for data
-                             in self.data.metadata] for key in keys])
+        values = numpy.array([self.get_metadata_values(kind=kind, key=key) for key in keys])
         # Unique values for each key.
         unique_values = [numpy.unique(value, return_counts=True) for value in values]
         dimensions = []
@@ -400,7 +402,7 @@ class Dataset():
         for i, value in enumerate(unique_values):
             if value[1][0] != dataset_size:
                 dimension = Dimension(kind, keys[i], tolerance=tolerance)
-                dimension.set_unique_values(numpy.unique(value[0]))
+                dimension.set_unique_values(value[0])
                 # Value that tells when does the change of value occur. It is used to know the order
                 # of the reshaping.
                 dimension.changing_value = self.__compute_changing_value(values[i])
@@ -411,9 +413,9 @@ class Dataset():
             _logger.info("Dimension {} of size {} has been added for reshaping"
                          .format(dimension.name, dimension.size))
 
-    def get_metadata_values(self, kind, key):
-        return numpy.unique([data.get_value(kind=kind, name=key)[0] for data
-                             in self.get_data().metadata])
+    def get_metadata_values(self, kind, key, indices=None, dimension=None):
+        return [data.get_value(kind=kind, name=key)[0] for data
+                in self.get_data(indices, dimension).metadata]
 
     def __compute_changing_value(self, values, changing_value=1):
         """
@@ -432,14 +434,10 @@ class Dataset():
 
         :returns: array_like
         """
-        if not self._dimensions_values:
-            data = self.get_data(indices).metadata
+        if not self._dimensions_values or indices is not None:
             for dimension in self.__dims:
-                values = numpy.empty((len(data)))
-                for row, metadata_frame in enumerate(data):
-                    values[row] = (metadata_frame.get_value(kind=dimension[1].kind,
-                                   name=dimension[1].name)[0])
-                self._dimensions_values[dimension[1].name] = values
+                self._dimensions_values[dimension[1].name] = self.get_metadata_values(
+                    kind=dimension[1].kind, key=dimension[1].name, indices=indices)
         return self._dimensions_values
 
     def compute_mosaicity_colorkey(self, scale=100):
@@ -524,7 +522,7 @@ class Dataset():
 
         if self._in_memory:
             new_data = background_subtraction(self.running_data, bg_data, method).view(Data)
-            new_data.save(_dir + "/data.hdf5")
+            new_data.save(os.path.join(_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             bg = numpy.zeros(self.running_data[0].shape, self.running_data.dtype)
@@ -582,7 +580,7 @@ class Dataset():
                 return
 
             urls = self.running_data.apply_funcs([(background_subtraction_2D, [bg])],
-                                                 save=_dir + "/data.hdf5", text="Applying background subtraction",
+                                                 save=os.path.join(_dir, "data.hdf5"), text="Applying background subtraction",
                                                  operation=Operation.BS)
             if urls is None:
                 return
@@ -629,11 +627,11 @@ class Dataset():
 
         if self._in_memory:
             new_data = hot_pixel_removal_3D(self.running_data, kernel).view(Data)
-            new_data.save(_dir + "/data.hdf5")
+            new_data.save(os.path.join(_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(hot_pixel_removal_2D, [kernel])],
-                                                 save=_dir + "/data.hdf5", text="Applying hot pixel removal",
+                                                 save=os.path.join(_dir, "data.hdf5"), text="Applying hot pixel removal",
                                                  operation=Operation.HP)
             if urls is None:
                 return
@@ -675,11 +673,11 @@ class Dataset():
 
         if self._in_memory:
             new_data = threshold_removal(self.running_data, bottom, top).view(Data)
-            new_data.save(_dir + "/data.hdf5")
+            new_data.save(os.path.join(_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(threshold_removal, [bottom, top])],
-                                                 save=_dir + "/data.hdf5", text="Applying threshold",
+                                                 save=os.path.join(_dir, "data.hdf5"), text="Applying threshold",
                                                  operation=Operation.THRESHOLD)
             if urls is None:
                 return
@@ -723,12 +721,12 @@ class Dataset():
 
         self.running_data = self.get_data(indices)
         if self._in_memory:
-            new_data = apply_3D_ROI(self.running_data, origin, size, center).view(Data)
-            new_data.save(roi_dir + "/data.hdf5", new_shape=new_data.shape)
+            new_data = apply_3D_ROI(self.running_data, origin, size, center).view(Data).copy()
+            new_data.save(os.path.join(roi_dir, "data.hdf5"), new_shape=new_data.shape)
         else:
             shape = numpy.append([self.running_data.shape[0]], apply_2D_ROI(self.running_data[0], origin, size, center).shape)
             urls = self.running_data.apply_funcs([(apply_2D_ROI, [origin, size, center])],
-                                                 save=roi_dir + "/data.hdf5", text="Applying roi",
+                                                 save=os.path.join(roi_dir, "data.hdf5"), text="Applying roi",
                                                  operation=Operation.ROI,
                                                  new_shape=shape)
             if urls is None:
@@ -941,8 +939,8 @@ class Dataset():
         first_size = (shape / (len(iterations) + 1)).astype(numpy.int)
         size = first_size
 
-        if not os.path.isdir(self.dir + "/waterfall/"):
-            os.mkdir(self.dir + "/waterfall/")
+        if not os.path.isdir(os.path.join(self.dir, "waterfall")):
+            os.mkdir(os.path.join(self.dir, "waterfall"))
 
         _logger.info("Starting waterfall NMF")
 
@@ -951,11 +949,11 @@ class Dataset():
             if indices is None:
                 indices = range(self.nframes)
             for j in indices:
-                filename = self.dir + "/waterfall/" + str(j) + ".npy"
+                filename = os.path.join(self.dir, "waterfall", str(j) + ".npy")
                 numpy.save(filename, resize(self.get_data(j), size))
                 new_urls.append(DataUrl(file_path=filename, scheme='fabio'))
             data = Data(new_urls, self.get_data(indices).metadata, self._in_memory)
-            dataset = Dataset(_dir=self.dir + "/waterfall/", data=data, in_memory=self._in_memory)
+            dataset = Dataset(_dir=os.path.join(self.dir, "waterfall"), data=data, in_memory=self._in_memory)
             if vstep:
                 v_step = vstep * (len(iterations) - i)
             if hstep:
@@ -968,9 +966,9 @@ class Dataset():
             H = H2
 
         try:
-            shutil.rmtree(self.dir + "/waterfall/")
+            shutil.rmtree(os.path.join(self.dir, "waterfall"))
         except Exception as e:
-            print('Failed to delete %s. Reason: %s' % (self.dir + "/waterfall/", e))
+            print('Failed to delete %s. Reason: %s' % (os.path.join(self.dir, "waterfall"), e))
 
         H = resize(H, (num_components, shape[0] * shape[1]))
 
@@ -997,7 +995,7 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/bss"
+        bss_dir = os.path.join(self.dir, "bss")
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
         if self._in_memory:
@@ -1029,7 +1027,7 @@ class Dataset():
             H, vals, W = model.H, model.singular_values, model.W
         return vals if return_vals else (H, W)
 
-    def nica(self, num_components, chunksize=None, num_iter=500, error_step=None, indices=None):
+    def nica(self, num_components, chunksize=None, num_iter=500, indices=None):
         """
         Compute Non-negative Independent Component Analysis on the data.
         The method, first converts, if not already, the data into an hdf5 file object
@@ -1049,7 +1047,7 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/bss/"
+        bss_dir = os.path.join(self.dir, "bss")
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
 
@@ -1057,7 +1055,7 @@ class Dataset():
             chunksize = None
         model = NICA(self.data.convert_to_hdf5(bss_dir), num_components, chunksize, indices=indices)
         try:
-            model.fit_transform(max_iter=num_iter, error_step=error_step)
+            model.fit_transform(max_iter=num_iter)
             return numpy.abs(model.H), numpy.abs(model.W)
         finally:
             self.data._file.close()
@@ -1090,13 +1088,13 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/bss/"
+        bss_dir = os.path.join(self.dir, "bss")
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
 
         if self._in_memory:
             from sklearn import decomposition
-            model = decomposition.NMF(n_components=num_components, init=init)
+            model = decomposition.NMF(n_components=num_components, init=init, max_iter=num_iter)
             if indices is not None:
                 X = self.data.convert_to_hdf5(bss_dir)[indices]
             else:
@@ -1119,7 +1117,7 @@ class Dataset():
 
             model = NMF(self.data.convert_to_hdf5(bss_dir), num_components, indices=indices)
             try:
-                model.fit_transform(max_iter=num_iter, H=H, W=W, vstep=vstep, hstep=hstep)
+                model.fit_transform(max_iter=num_iter, H=H, W=W, vstep=vstep, hstep=hstep, error_step=error_step)
                 return model.H, model.W
             finally:
                 self.data._file.close()
@@ -1130,7 +1128,7 @@ class Dataset():
         Applies both NICA and NMF to the data. The init H and W for NMF are the
         result of NICA.
         """
-        H, W = self.nica(num_components, chunksize, num_iter, error_step, indices=indices)
+        H, W = self.nica(num_components, chunksize, num_iter, indices=indices)
 
         # Initial NMF factorization: X = F0 * G0
         W = numpy.abs(W)
@@ -1151,12 +1149,11 @@ class Dataset():
         """
 
         self.running_data = self.get_data(indices)
-        data = self.get_data()
         if indices is None:
             indices = range(self.nframes)
         for axis, dim in self.dims:
             # Get motor values per image of the stack
-            values = numpy.array([data.metadata[i].get_value(kind=dim.kind, name=dim.name) for i in indices])
+            values = self.get_dimensions_values(indices)[dim.name]
             if self._in_memory:
                 # Data in memory
                 com, std, skew, kurt = compute_moments(values, self.running_data)
@@ -1214,7 +1211,7 @@ class Dataset():
         """
 
         _dir = self.dir if _dir is None else _dir
-        _dir += "/fit"
+        _dir = os.path.join(_dir, "fit")
 
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
@@ -1249,7 +1246,7 @@ class Dataset():
                             c_images = p.map(partial(chunk_image, start, chunk_shape), data)
                         fitted_data = fit_data(numpy.asarray(c_images), int_thresh=int_thresh)
                         for k in range(len(fitted_data)):
-                            filename = _dir + "/data_fit_" + str(indices[k]).zfill(4) + ".npy"
+                            filename = os.path.join(_dir, "data_fit_" + str(indices[k]).zfill(4) + ".npy")
                             if (i, j) != (0, 0):
                                 # If chunk is not the first, load image from disk
                                 img = numpy.load(filename)
@@ -1268,7 +1265,7 @@ class Dataset():
             else:
                 fitted_data = fit_data(data, int_thresh=int_thresh, _tqdm=True)
                 for i, image in enumerate(fitted_data):
-                    filename = _dir + "/data_fit" + str(indices[i]).zfill(4) + ".npy"
+                    filename = os.path.join(_dir, "data_fit" + str(indices[i]).zfill(4) + ".npy")
                     numpy.save(filename, image)
                     urls.append(DataUrl(file_path=filename, scheme='fabio'))
         else:
@@ -1279,7 +1276,7 @@ class Dataset():
                     if len(data) < 3:
                         _logger.warning("Not enough values for fitting")
                         break
-                    values = numpy.array([data.metadata[i].get_value(kind=dim.kind, name=dim.name) for i in range(len(data))]).reshape(-1)
+                    values = numpy.array(self.get_metadata_values(kind=dim.kind, key=dim.name, indices=indices, dimension=[i, j])).reshape(-1)
                     if not data.in_memory:
                         # Use chunks to load the data into memory
                         start = [0, 0]
@@ -1297,7 +1294,7 @@ class Dataset():
                                     c_images = p.map(partial(chunk_image, start, chunk_shape), data)
                                 fitted_data = fit_data(numpy.asarray(c_images), values=values, int_thresh=int_thresh)
                                 for k in range(len(fitted_data)):
-                                    filename = _dir + "/data_fit_" + self.dims.get(i).name + str(rindices[k]).zfill(4) + ".npy"
+                                    filename = os.path.join(_dir, "data_fit_" + self.dims.get(i).name + str(rindices[k]).zfill(4) + ".npy")
                                     if (chunk1, chunk2) != (0, 0):
                                         # If chunk is not the first, load image from disk
                                         img = numpy.load(filename)
@@ -1318,7 +1315,7 @@ class Dataset():
                             return
                         fitted_data = fit_data(data, values=values, int_thresh=int_thresh, _tqdm=True)
                         for k, image in enumerate(fitted_data):
-                            filename = _dir + "/data_fit_" + self.dims.get(dimension[0][0]).name + str(rindices[k]).zfill(4) + ".npy"
+                            filename = os.path.join(_dir, "data_fit_" + self.dims.get(dimension[0][0]).name + str(rindices[k]).zfill(4) + ".npy")
                             numpy.save(filename, image)
                             urls.append(DataUrl(file_path=filename, scheme='fabio'))
 
@@ -1335,7 +1332,7 @@ class Dataset():
         data = Data(new_urls.reshape(self.data.urls.shape), self.data.metadata, in_memory=True)  # to modify
         return Dataset(_dir=_dir, data=data, dims=self.__dims, transformation=self.transformation, in_memory=True)
 
-    def compute_transformation(self, d, kind="magnification", rotate=False, topography=[False, 0]):
+    def compute_transformation(self, d, kind="magnification", rotate=False, topography=[False, 0], center=True):
 
         """
         Computes transformation matrix.
@@ -1367,10 +1364,11 @@ class Dataset():
             self.transformation = Transformation(kind, transformation, rotate)
         else:
             obx = self.get_metadata_values(POSITIONER_METADATA, "obx")[0]
-            obpitch = self.get_metadata_values(POSITIONER_METADATA, "obpitch")
+            obpitch = numpy.unique(self.get_metadata_values(POSITIONER_METADATA, "obpitch"))
             obpitch = obpitch[len(obpitch) // 2]
             mainx = -self.get_metadata_values(POSITIONER_METADATA, "mainx")[0]
-            self.transformation = Transformation(kind, compute_magnification(H, W, d, obx, obpitch, mainx, topography), rotate)
+            self.transformation = Transformation(kind, compute_magnification(
+                H, W, d, obx, obpitch, mainx, topography, center), rotate)
 
     def __deepcopy__(self, memo):
         """
@@ -1400,10 +1398,11 @@ class Data(numpy.ndarray):
         if in_memory:
             if data is None or urls.shape != data.shape[:-2]:
                 # Create array as stack of images
-                input_data = []
-                for url in urls.flatten():
-                    input_data.append(utils.get_data(url))
-                data = numpy.asarray(input_data)
+                # Create array as stack of images
+                img = utils.get_data(urls.flatten()[0])
+                data = numpy.empty((urls.flatten().shape[0], img.shape[0], img.shape[1]), img.dtype)
+                for i in range(len(urls.flatten())):
+                    data[i] = utils.get_data(urls.flatten()[i])
             shape = list(urls.shape)
             shape.append(data.shape[-2])
             shape.append(data.shape[-1])
@@ -1634,8 +1633,10 @@ class Data(numpy.ndarray):
         :return: Hdf5 file
         :rtype: `h5py.File`
         """
-
-        self._file = h5py.File(_dir + '/data.hdf5', 'a')
+        try:
+            self._file = h5py.File(_dir + '/data.hdf5', 'a')
+        except OSError:
+            self._file = h5py.File(os.path.join(_dir, 'data.hdf5'), 'w')
         data = self.flatten()
         shape = data[0].shape[0] * data[0].shape[1]
         if "dataset" in self._file:
