@@ -26,14 +26,14 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "04/03/2020"
+__date__ = "07/07/2021"
 
 
 import enum
 import numpy
 import cv2
 from scipy.ndimage import center_of_mass, fourier_shift
-from skimage import feature
+from skimage import registration
 
 from silx.utils.enum import Enum as _Enum
 
@@ -59,7 +59,7 @@ def compute_com(data):
     :param numpy.ndarray data: stack of images.
     :returns: the vector of intensities and the com.
     """
-    intensity = data.sum(axis=1).sum(axis=1)
+    intensity = data.sum(axis=1)
     return intensity, int(center_of_mass(intensity)[0])
 
 
@@ -72,15 +72,15 @@ def diff_com(img1, img2):
     return numpy.array(center_of_mass(img2)) - numpy.array(center_of_mass(img1))
 
 
-def find_shift(img1, img2, upsampling_factor=1000):
+def find_shift(img1, img2, upsample_factor=1000):
     """
     Uses the function ``register_translation`` from skimage to find the shift between two images.
 
     :param array_like img1: first image.
     :param array_like img2: second image, must be same dimensionsionality as ``img1``.
-    :param int upsampling_factor: optional.
+    :param int upsample_factor: optional.
     """
-    return feature.register_translation(img1, img2, upsampling_factor, return_error=False)
+    return registration.phase_cross_correlation(img1, img2, upsample_factor=upsample_factor, return_error=False)
 
 
 def _numpy_fft_shift(img, dx, dy):
@@ -169,18 +169,15 @@ def normalize(x):
     return x / numpy.linalg.norm(x)
 
 
-def improve_linear_shift(data, v, h_max, h_step, nimages=None, shift_approach="linear"):
+def improve_linear_shift(data, v, h, epsilon, steps, nimages=None, shift_approach="linear"):
     """
-    Function to find the best shift between the images. It loops ``h_max * h_step`` times,
+    Function to find the best shift between the images. It loops ``steps`` times,
     applying a different shift each, and trying to find the one that has the best result.
 
     :param array_like data: The stack of images.
     :param 2-dimensional array_like v: The vector with the direction of the shift.
-    :param number h_max: The maximum value that h can achieve, being h the shift between
-        images divided by the vector v (i.e the coordinates of the shift in base v).
-    :param number h_step: Spacing between the ``h`` tried. For any `` shift = h * v * idx``,
-        where ``idx`` is the index of the image to apply the shift to, this is the distance
-        between two adjacent values of h.
+    :param float epsilon: Maximum value of h
+    :param int steps: Number of different tries of h.
     :param int nimages: The number of images to be used to find the best shift. It has to
         be smaller or equal as the length of the data. If it is smaller, the images used
         are chosen using `numpy.random.choice`, without replacement.
@@ -195,23 +192,23 @@ def improve_linear_shift(data, v, h_max, h_step, nimages=None, shift_approach="l
         iData = numpy.random.choice(iData, nimages, False)
 
     score = {}
-    utils.advancement_display(0, h_max, "Finding shift")
-    for h in numpy.arange(0, h_max, h_step):
+    utils.advancement_display(0, h + epsilon, "Finding shift")
+    step = epsilon / steps
+    for h_ in numpy.arange(0, h + epsilon, step):
         result = numpy.zeros(data[0].shape)
         for iFrame in iData:
-            shift = h * v * iFrame
+            shift = h_ * v * iFrame
             result += apply_shift(data[iFrame], shift, shift_approach)
 
         # Compute score using normalized variance
         # TODO: add more autofocus options
-        score[h] = normalized_variance(result)
-        utils.advancement_display(h + h_step, h_max, "Finding shift")
-
+        score[h_] = normalized_variance(result)
+        utils.advancement_display(h_ + step, h + epsilon, "Finding shift")
     optimal_h = max(score.keys(), key=(lambda k: score[k]))
     return optimal_h
 
 
-def shift_detection(data, h_max=0.5, h_step=0.01):
+def shift_detection(data, steps, shift_approach="linear"):
     """
     Finds the linear shift from a set of images.
 
@@ -232,7 +229,12 @@ def shift_detection(data, h_max=0.5, h_step=0.01):
             second_sum += data[i]
     shift = find_shift(first_sum, second_sum, 1000)
     v = normalize(shift)
-    h = improve_linear_shift(data, v, h_max, h_step)
+    v_ = 2 * shift / len(data)
+    h = numpy.sqrt(v_[0]**2 + v_[1]**2)
+    if not h:
+        return numpy.outer([0, 0], numpy.arange(len(data)))
+    epsilon = 2 * h
+    h = improve_linear_shift(data, v, h, epsilon, steps, shift_approach=shift_approach)
     return numpy.outer(h * v, numpy.arange(len(data)))
 
 
@@ -312,14 +314,13 @@ def random_search(data, optimal_shift, iterations, sigma=None, shift_approach='l
 
     utils.advancement_display(0, iterations)
     for i in range(iterations):
-        result = numpy.zeros(data[0].shape)
+        result = []
         normal = numpy.random.multivariate_normal((0, 0), sigma * numpy.eye(2), len(data)).T
         n_shift = optimal_shift + normal
         for iFrame in range(len(data)):
             shifty = n_shift[0][iFrame]
             shiftx = n_shift[1][iFrame]
-            result += data.apply_funcs([(apply_shift, [[shifty, shiftx], shift_approach])],
-                                       indices=iFrame, return_data=True)[0]
+            result += [apply_shift(data[iFrame], [shifty, shiftx], shift_approach)]
         score = normalized_variance(result)
         if best_score < score:
             best_result = n_shift
