@@ -107,6 +107,7 @@ def fit_rocking_curve(y_values, values=None, num_points=None, int_thresh=None):
         pars, cov = curve_fit(f=gaussian, xdata=x, ydata=y, p0=p0)
         y_gauss = gaussian(numpy.linspace(x[0], x[-1], num_points), *pars)
         y_gauss[numpy.isnan(y_gauss)] = 0
+        y_gauss[y_gauss < 0] = 0
         return y_gauss
     except RuntimeError:
         return y
@@ -123,9 +124,9 @@ def fit_data(data, moments=None, values=None, int_thresh=15, _tqdm=False):
     cpus = multiprocessing.cpu_count()
     with Pool(cpus - 1) as p:
         if _tqdm and tqdm is not None:
-            new_data = list(tqdm.tqdm(p.imap(partial(fit_rocking_curve, values=values, int_thresh=15), g), total=data.shape[1] * data.shape[2]))
+            new_data = list(tqdm.tqdm(p.imap(partial(fit_rocking_curve, values=values, int_thresh=int_thresh), g), total=data.shape[1] * data.shape[2]))
         else:
-            new_data = p.map(partial(fit_rocking_curve, values=values, int_thresh=15), g)
+            new_data = p.map(partial(fit_rocking_curve, values=values, int_thresh=int_thresh), g)
     return numpy.array(new_data).T.reshape(data.shape)
 
 
@@ -150,6 +151,28 @@ def compute_moments(values, data):
     return com, std, skews, kurt
 
 
+def compute_peak_position(data, values=None, center_data=False):
+    """
+    Compute peak position map
+
+    :param bool center_data: If True, the values are centered on 0.
+    """
+    if values is not None:
+        x = numpy.array(numpy.argmax(data, axis=0))
+        if center_data:
+            middle = float(min(values) + values.ptp()) / 2
+            values -= middle
+        image = [values[i] for i in x.flatten()]
+        image = numpy.reshape(image, x.shape)
+    else:
+        image = numpy.array(numpy.argmax(data, axis=0))
+        if center_data:
+            middle = len(data) / 2
+            vals = numpy.linspace(-middle, middle, len(data))
+            image = image * numpy.ptp(vals) / len(data) + numpy.min(vals)
+    return image
+
+
 def compute_rsm(H, W, d, ffz, mainx):
     """
     :param int H: height of the image in pixels.
@@ -170,7 +193,7 @@ def compute_rsm(H, W, d, ffz, mainx):
     return pix_arr
 
 
-def compute_magnification(H, W, d, obx, obpitch, mainx, topography=[False, 0]):
+def compute_magnification(H, W, d, obx, obpitch, mainx, topography=[False, 0], center=True):
     """
     :param int H: height of the image in pixels.
     :param int W: width of the image in pixels.
@@ -188,8 +211,12 @@ def compute_magnification(H, W, d, obx, obpitch, mainx, topography=[False, 0]):
     d2 = mainx / numpy.cos(numpy.radians(obpitch)) - d1
     M = d2 / d1
     d /= M
-    pix_arr[0] = (pix_arr[0] - W / 2) * d
-    pix_arr[1] = (H / 2 - pix_arr[1]) * d
+    if center:
+        pix_arr[0] = (pix_arr[0] - W / 2) * d
+        pix_arr[1] = (H / 2 - pix_arr[1]) * d
+    else:
+        pix_arr[0] = pix_arr[0] * d
+        pix_arr[1] = (H - 1 - pix_arr[1]) * d
     if topography[0]:
         pix_arr[topography[1]] /= numpy.sin(numpy.radians(obpitch))
     return pix_arr
