@@ -28,6 +28,7 @@ __authors__ = ["J. Garriga"]
 __license__ = "MIT"
 __date__ = "09/12/2020"
 
+import os
 import copy
 import unittest
 import numpy
@@ -412,9 +413,11 @@ class TestDataset(unittest.TestCase):
 
     def test_apply_fit_in_memory(self):
         """ Tests fitting data in memory"""
-        new_dataset = self.dataset.apply_fit()
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10,
+                                            header=True, _dir=self._dir)
+        new_dataset = dataset.apply_fit()
 
-        self.assertEqual(new_dataset.nframes, self.dataset.nframes)
+        self.assertEqual(new_dataset.nframes, dataset.nframes)
         self.assertNotEqual(new_dataset.data.urls[0], self.dataset.data.urls[0])
 
     def test_apply_fit_in_disk(self):
@@ -428,6 +431,23 @@ class TestDataset(unittest.TestCase):
         self.assertNotEqual(fit_dataset.data.urls[0], dataset.data.urls[0])
         numpy.testing.assert_equal(fit_dataset.data[5], dataset.data[5])
 
+    def test_compute_rsm_in_memory(self):
+        """ Tests fitting data in memory"""
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10,
+                                            header=True, _dir=self._dir)
+        dataset.compute_transformation(d=0.1, kind='rsm')
+
+        self.assertEqual(dataset.transformation.transformation[0].shape, dataset.get_data(0).shape)
+
+    def test_apply_rsm_in_disk(self):
+        """Tests fitting data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10,
+                                            header=True, in_memory=False)
+        dataset.compute_transformation(d=0.1, kind='rsm')
+
+        self.assertEqual(dataset.transformation.transformation[1].shape, dataset.get_data(0).shape)
+
 
 class TestData(unittest.TestCase):
 
@@ -521,7 +541,7 @@ class TestData(unittest.TestCase):
     def test_save(self):
         data = numpy.random.rand(10, 100, 100).view(Data)
         # data.in_memory = True
-        data.save(self._dir + "/data.hdf5")
+        data.save(os.path.join(self._dir, "data.hdf5"))
 
         numpy.testing.assert_array_equal(data.shape, (10, 100, 100))
 
