@@ -48,7 +48,7 @@ class TestDimension(unittest.TestCase):
         """
         self._dir = tempfile.mkdtemp()
         counter_mne = "a b c d e f g h"
-        motor_mne = "x y z k h m n"
+        motor_mne = "obpitch y z mainx ffz m obx"
         dims = (20, 100, 100)
         # Create headers
         header = []
@@ -71,8 +71,10 @@ class TestDimension(unittest.TestCase):
                     header[i]["motor_pos"] += str(a[int((i > 4 and i < 10) or i > 14)]) + " "
                 elif m == "m":
                     header[i]["motor_pos"] += str(b[i % 5]) + " "
-                elif m == "x":
+                elif m == "obpitch":
                     header[i]["motor_pos"] += str(c[int(i > 9)]) + " "
+                elif m == "mainx":
+                    header[i]["motor_pos"] += "50 "
                 else:
                     header[i]["motor_pos"] += str(motors[j]) + " "
 
@@ -168,14 +170,14 @@ class TestDimension(unittest.TestCase):
         self.assertEqual(self.in_memory_dataset.dims.ndim, 3)
         self.assertEqual(self.in_memory_dataset.dims.get(0).name, "m")
         self.assertEqual(self.in_memory_dataset.dims.get(1).name, "z")
-        self.assertEqual(self.in_memory_dataset.dims.get(2).name, "x")
+        self.assertEqual(self.in_memory_dataset.dims.get(2).name, "obpitch")
 
         # In disk
         self.in_disk_dataset.find_dimensions(POSITIONER_METADATA)
         self.assertEqual(self.in_disk_dataset.dims.ndim, 3)
         self.assertEqual(self.in_disk_dataset.dims.get(0).name, "m")
         self.assertEqual(self.in_disk_dataset.dims.get(1).name, "z")
-        self.assertEqual(self.in_disk_dataset.dims.get(2).name, "x")
+        self.assertEqual(self.in_disk_dataset.dims.get(2).name, "obpitch")
 
     def test_reshaped_data(self):
         """ Tests the correct reshaping of the data """
@@ -330,5 +332,37 @@ class TestDimension(unittest.TestCase):
         self.in_disk_dataset.clear_dims()
         self.assertEqual(self.in_disk_dataset.dims.ndim, 0)
 
+    def test_apply_moments_in_memory(self):
+        """ Tests finding moments"""
+
+        # In memory
+        self.in_memory_dataset.find_dimensions(POSITIONER_METADATA)
+        dataset = self.in_memory_dataset.reshape_data()
+        moments = dataset.apply_moments(indices=[1, 2, 3, 4])
+        self.assertEqual(moments[0][0].shape, dataset.get_data(0).shape)
+        self.assertEqual(moments[1][3].shape, dataset.get_data(0).shape)
+
+        #  In disk
+        self.in_disk_dataset.find_dimensions(POSITIONER_METADATA)
+        dataset = self.in_disk_dataset.reshape_data()
+        moments = dataset.apply_moments(indices=[1, 2, 3, 4])
+        self.assertEqual(moments[0][0].shape, dataset.get_data(0).shape)
+        self.assertEqual(moments[1][3].shape, dataset.get_data(0).shape)
+
+    def test_compute_magnification(self):
+        """ Tests fitting data in memory"""
+
+        # In memory
+        self.in_memory_dataset.find_dimensions(POSITIONER_METADATA)
+        dataset = self.in_memory_dataset.reshape_data()
+        dataset.compute_transformation(d=0.1)
+        self.assertEqual(dataset.transformation.transformation[0].shape, dataset.get_data(0).shape)
+
+        #  In disk
+        self.in_disk_dataset.find_dimensions(POSITIONER_METADATA)
+        dataset = self.in_disk_dataset.reshape_data()
+        dataset.compute_transformation(d=0.1)
+        self.assertEqual(dataset.transformation.transformation[1].shape, dataset.get_data(0).shape)
+
     def tearDown(self):
         shutil.rmtree(self._dir)
