@@ -106,12 +106,13 @@ class BSSWidget(qt.QMainWindow):
     def showButton(self):
         self._computeB.show()
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter.
 
         :param Dataset dataset: dataset
         """
+        self._parent = parent
         self.dataset = dataset
         self.indices = indices
         self.bg_indices = bg_indices
@@ -119,6 +120,10 @@ class BSSWidget(qt.QMainWindow):
         self.computeButton.setEnabled(True)
         self.detectButton.setEnabled(True)
 
+    def _updateDataset(self, widget, dataset):
+        self._parent._updateDataset(widget, dataset)
+        self.dataset = dataset
+
     def _computeBSS(self):
         """
         Computes blind source separation with the chosen method.
@@ -131,17 +136,19 @@ class BSSWidget(qt.QMainWindow):
         self.sigComputed.emit(method, n_comp)
         if method == Method.PCA:
             self._thread = OperationThread(self, self.dataset.pca)
+            self._thread.setArgs(n_comp, indices=self.indices)
         elif method == Method.NICA:
             self._thread = OperationThread(self, self.dataset.nica)
+            self._thread.setArgs(n_comp, indices=self.indices, error_step=5)
         elif method == Method.NMF:
             self._thread = OperationThread(self, self.dataset.nmf)
+            self._thread.setArgs(n_comp, indices=self.indices, error_step=5)
         elif method == Method.NICA_NMF:
             self._thread = OperationThread(self, self.dataset.nica_nmf)
+            self._thread.setArgs(n_comp, indices=self.indices, error_step=5)
         else:
             raise ValueError('BSS method not managed')
 
-        # Set arguments and start thread
-        self._thread.setArgs(n_comp, indices=self.indices)
         self._thread.finished.connect(self._displayComponents)
         self._thread.start()
 
