@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "11/12/2020"
+__date__ = "26/04/2021"
 
 import numpy
 
@@ -118,14 +118,13 @@ class DisplayComponentsWidget(qt.QMainWindow):
         :param array_like W: array with the rocking curves intensity
         :param dict dimensions: dictionary with the values of the dimensions
         """
-        self.components = components
+        self.components = numpy.round(components, 5)
         self.W = W
         self.dimensions = dimensions
         self.dimensions_shape = shape
         self._activateCurvesPlot(True)
         self._sv_components.setStack(self.components)
         self._plot_rocking_curves.getPlot().clear()
-        self._plot_rocking_curves.getPlot().setGraphYLabel("Values")
         for i in range(len(self.W.T)):
             self._plot_rocking_curves.getPlot().addCurve(numpy.arange(len(self.W.T[i])),
                                                          self.W.T[i], legend=str(i))
@@ -135,8 +134,8 @@ class DisplayComponentsWidget(qt.QMainWindow):
             self._scatter_rocking_curves.getPlotWidget().setGraphXLabel(keys[0])
             self._scatter_rocking_curves.getPlotWidget().setGraphYLabel(keys[1])
             self._scatter_rocking_curves.setData(
-                self.dimensions[keys[0]].astype(numpy.float),
-                self.dimensions[keys[1]].astype(numpy.float), self.W.T[0])
+                numpy.asanyarray(self.dimensions[keys[0]], dtype=float),
+                numpy.asanyarray(self.dimensions[keys[1]], dtype=float), self.W.T[0])
             self._scatter_rocking_curves.setColormap(colormap)
             self._scatter_rocking_curves.resetZoom()
 
@@ -206,23 +205,25 @@ class DisplayComponentsWidget(qt.QMainWindow):
         :param int axis: 0 if no motor is chosen, else axis of the motor.
         """
         if axis == 0:
-            self._plot_rocking_curves.getPlot().setGraphYLabel("Values")
             # Whole rocking curves are showed
             self._plot_rocking_curves.clear()
             for i in range(len(self.W.T)):
                 self._plot_rocking_curves.addCurve(numpy.arange(len(self.W.T[i])), self.W.T[i], legend=str(i))
             self._plot_rocking_curves.getBrowser().setEnabled(False)
+            self._plot_rocking_curves.getPlot().setGraphXLabel("Image id")
         elif axis != -1 and axis is not None:
-            text = self._plot_rocking_curves.getDimensionsCB().currentText()
-            self._plot_rocking_curves.getPlot().setGraphYLabel(text)
             self.dimension = len(self.dimensions) - axis
             self._plot_rocking_curves.clear()
             self._plot_rocking_curves.getBrowser().setEnabled(True)
             self._plot_rocking_curves.getBrowser().setRange(0, self.dimensions_shape[self.dimension] - 1)
             self._plot_rocking_curves.getBrowser().setValue(0)
+            text = list(self.dimensions.keys())[self.dimension]
+            self._plot_rocking_curves.getPlot().setGraphXLabel(text)
+            dim_values = list(self.dimensions.values())[self.dimension]
+            dim_values = numpy.take(dim_values.reshape(self.dimensions_shape), self.index, self.dimension)
             for i in range(len(self.W.T)):
                 W = numpy.take(self.W.T[i].reshape(self.dimensions_shape), self.index, self.dimension)
-                self._plot_rocking_curves.addCurve(numpy.arange(len(W)), W, legend=str(i))
+                self._plot_rocking_curves.addCurve(dim_values, W, legend=str(i))
         else:
             raise ValueError('Axis %s is not managed' % axis)
 
@@ -233,9 +234,11 @@ class DisplayComponentsWidget(qt.QMainWindow):
         """
         self.index = index
         self._plot_rocking_curves.clear()
+        dim_values = list(self.dimensions.values())[self.dimension]
+        dim_values = numpy.take(dim_values.reshape(self.dimensions_shape), self.index, self.dimension)
         for i in range(len(self.W.T)):
             W = numpy.take(self.W.T[i].reshape(self.dimensions_shape), self.index, self.dimension)
-            self._plot_rocking_curves.addCurve(numpy.arange(len(W)), W, legend=str(i))
+            self._plot_rocking_curves.addCurve(dim_values, W, legend=str(i))
 
     def setStackViewColormap(self, colormap):
         """
