@@ -38,7 +38,7 @@ from .utils import ChooseDimensionDock
 from silx.utils.enum import Enum as _Enum
 
 import darfix
-from darfix.core.mapping import fit_rocking_curve
+from darfix.core.mapping import fit_rocking_curve, compute_peak_position
 from darfix.core.dataset import Operation
 from .operationThread import OperationThread
 
@@ -52,6 +52,7 @@ class Method(_Enum):
     INTENSITY = "Integrated intensity"
     FWHM = "FWHM"
     PEAK = "Peak position"
+    RESIDUALS = "Residuals"
 
 
 class RockingCurvesWidget(qt.QMainWindow):
@@ -66,7 +67,7 @@ class RockingCurvesWidget(qt.QMainWindow):
         widget = qt.QWidget(parent=self)
         layout = qt.QGridLayout()
 
-        self._sv = StackView(parent=self)
+        self._sv = StackView(parent=self, position=True)
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME))
         self._plot = Plot1D(parent=self)
         self._plot.setGraphTitle("Rocking curves")
@@ -112,12 +113,13 @@ class RockingCurvesWidget(qt.QMainWindow):
         widget.setLayout(layout)
         self.setCentralWidget(widget)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter.
 
         :param Dataset dataset: dataset
         """
+        self._parent = parent
         self.dataset = dataset
         self.indices = indices
         self._update_dataset = dataset
@@ -260,28 +262,16 @@ class RockingCurvesWidget(qt.QMainWindow):
         method = Method(method)
         data = self._update_dataset.get_data(self.indices)
         if method == Method.INTENSITY:
-            self._plotMaps.addImage(data.sum(axis=0))
+            self._plotMaps.addImage(self._update_dataset.zsum())
         elif method == Method.FWHM:
             std = numpy.std(data, axis=0)
             std[numpy.isnan(std)] = 0
             self._plotMaps.addImage(darfix.config.FWHM_VAL * std)
         elif method == Method.PEAK:
-            if self._dimension is not None:
-                axis = self.dataset.dims.ndim - self._dimension[0][0] - 1
-                dim_vals = numpy.copy(self.dataset.dims.get(axis).unique_values)
-                x = numpy.argmax(data, axis=0)
-                if self._centerDataCheckbox.isChecked():
-                    middle = float(dim_vals[0]) + (float(dim_vals[-1]) - float(dim_vals[0])) / 2
-                    dim_vals -= middle
-                image = x * numpy.ptp(dim_vals) / len(data) + min(dim_vals)
-                self._plotMaps.addImage(image)
-            else:
-                x = numpy.argmax(data, axis=0)
-                if self._centerDataCheckbox.isChecked():
-                    middle = len(data) / 2
-                    vals = numpy.linspace(-middle, middle, len(data))
-                    x = x * numpy.ptp(vals) / len(data) + numpy.min(vals)
-                self._plotMaps.addImage(x)
+            self._plotMaps.addImage(self.com)
+        elif method == Method.RESIDUALS:
+            self._plotMaps.addImage(numpy.sqrt(numpy.subtract(
+                self._update_dataset.zsum(self.indices), self.dataset.zsum(self.indices)) ** 2))
 
     def __abort(self):
         self._abortFit.setEnabled(False)
@@ -297,6 +287,14 @@ class RockingCurvesWidget(qt.QMainWindow):
         if self._thread.data:
             self._update_dataset = self._thread.data
             assert self._update_dataset is not None
+            values = None
+            if self._dimension is not None:
+                axis = self.dataset.dims.ndim - self._dimension[0][0] - 1
+                dim = self.dataset.dims.get(axis)
+                values = self._update_dataset.get_metadata_values(kind=dim.kind, key=dim.name, indices=self.indices)
+            self.com = compute_peak_position(self._update_dataset.get_data(self.indices), values,
+                                             center_data=self._centerDataCheckbox.isChecked())
+
             self._updatePlot(self._methodCB.currentText())
             self._plotMaps.show()
             self._methodCB.show()
