@@ -51,24 +51,31 @@ def assert_string(s, enums):
 
 
 # Print iterations progress
-def advancement_display(iteration, total, prefix='', suffix='', decimals=1, length=100, fill='█'):
+def advancement_display(ncurrent, ntotal, prefix='', suffix='', decimals=1, length=100, fill='█', left="-"):
     """
     Call in a loop to create terminal progress bar
     @params:
-        iteration   - Required  : current iteration (Int)
-        total       - Required  : total iterations (Int)
-        prefix      - Optional  : prefix string (Str)
-        suffix      - Optional  : suffix string (Str)
-        decimals    - Optional  : positive number of decimals in percent complete (Int)
-        length      - Optional  : character length of bar (Int)
-        fill        - Optional  : bar fill character (Str)
+        ncurrent   - Required  : current N (Int)
+        ntotal     - Required  : total expected N (Int)
+        prefix     - Optional  : prefix string (Str)
+        suffix     - Optional  : suffix string (Str)
+        decimals   - Optional  : positive number of decimals in percent complete (Int)
+        length     - Optional  : character length of bar (Int)
+        fill       - Optional  : bar fill character (Str)
+        left       - Optional  : bar fill character (Str)
     """
-    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
-    filledLength = int(length * iteration // total)
-    bar = fill * filledLength + '-' * (length - filledLength)
+    if ntotal:
+        progress = ncurrent / float(ntotal)
+    else:
+        progress = ncurrent + 1
+    percent_fmt = "{0:." + str(decimals) + "f}"
+    percent = percent_fmt.format(100 * progress)
+    filledLength = int(length * min(progress, 1))
+    leftLength = length - filledLength
+    bar = fill * filledLength + left * leftLength
     print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end='\r')
     # Print New Line on Complete
-    if iteration == total:
+    if ncurrent >= ntotal:
         print()
 
 
