@@ -58,7 +58,7 @@ try:
     from Orange.canvas.registry import set_global_registry
     from Orange.canvas.scheme import SchemeNode, SchemeLink
     from Orange.canvas.registry import cache
-except:
+except ImportError:
     pass
 
 from unittest import TestCase
