@@ -32,7 +32,7 @@ __date__ = "12/08/2019"
 import copy
 
 from Orange.widgets.widget import OWWidget, Input, Output
-from darfix.core.dataset import Dataset
+from darfix.core.process import DataCopy
 
 
 class DataCopyWidgetOW(OWWidget):
@@ -44,23 +44,29 @@ class DataCopyWidgetOW(OWWidget):
     name = "data copy"
     icon = "icons/copy.svg"
     want_main_area = False
+    ewokstaskclass = DataCopy
 
     # Inputs
     class Inputs:
-        dataset = Input("dataset", Dataset)
+        dataset = Input("dataset", tuple)
 
     # Outputs
     class Outputs:
-        dataset = Output("dataset", Dataset)
+        dataset = Output("dataset", tuple)
 
     def __init__(self):
         super().__init__()
 
     @Inputs.dataset
-    def setDataset(self, dataset):
+    def setDataset(self, _input):
         # Copy and send new dataset
-        cp_dataset = copy.deepcopy(dataset)
-        self.Outputs.dataset.send(cp_dataset)
+        dataset, update = _input
+        if not update:
+            self.cp_dataset = copy.deepcopy(dataset[1:])
+            self.Outputs.dataset.send(((self,) + self.cp_dataset, None))
+
+    def _updateDataset(self, widget, dataset):
+        self.Outputs.dataset.send(((self, dataset) + self.cp_dataset[1:], widget))
 
     def setVisible(self, visible):
         pass
