@@ -37,6 +37,7 @@ from silx.gui import qt
 
 from darfix.gui.operationThread import OperationThread
 from darfix.gui.datasetSelectionWidget import DatasetSelectionWidget
+from darfix.core.process import DataSelection
 
 
 class DataSelectionWidgetOW(OWWidget):
@@ -47,10 +48,11 @@ class DataSelectionWidgetOW(OWWidget):
     name = "data selection"
     icon = "icons/upload.svg"
     want_main_area = False
+    ewokstaskclass = DataSelection
 
     # Outputs
     class Outputs:
-        dataset = Output("dataset", tuple)
+        dataset = Output("dataset", tuple, bool)
 
     # Settings
     filenames = Setting(list(), schema_only=True)
@@ -97,6 +99,10 @@ class DataSelectionWidgetOW(OWWidget):
         self.__updatingData = True
         self.information("Downloading dataset")
 
+    def _updateDataset(self, widget, dataset):
+        self._widget._updateDataset(dataset)
+        self.Outputs.dataset.send(((self,) + self._widget.getDataset(), widget))
+
     def _sendSignal(self):
         """
         Function to emit the new dataset.
@@ -105,8 +111,9 @@ class DataSelectionWidgetOW(OWWidget):
         self._thread.finished.disconnect(self._sendSignal)
         self.__updatingData = False
         self.information()
-        self.Outputs.dataset.send(self._widget.getDataset())
-        self.close()
+        if self._thread.data:
+            self.Outputs.dataset.send(((self,) + self._widget.getDataset(), None))
+            self.close()
 
     def updateSettings(self):
         """
