@@ -29,13 +29,14 @@ __date__ = "24/09/2019"
 
 
 from silx.gui import qt
-from Orange.widgets import gui
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
+from darfix.core import utils
 from darfix.core.dataset import Dimension
 from darfix.gui.dimensionsWidget import DimensionWidget
 from functools import partial
 import logging
+from darfix.core.process import DimensionDefinition
 
 _logger = logging.getLogger(__file__)
 
@@ -45,10 +46,12 @@ class DimensionWidgetOW(OWWidget):
     Widget used to define the calibration of the experimentation (select motor
     positions...)
     """
+
     name = "dimension definition"
     id = "orange.widgets.darfix.dimensiondefinition"
     description = "Define the dimension followed during the acquisition"
     icon = "icons/param_dims.svg"
+    ewokstaskclass = DimensionDefinition
 
     priority = 4
     category = "esrfWidgets"
@@ -62,23 +65,20 @@ class DimensionWidgetOW(OWWidget):
     class Outputs:
         dataset = Output("dataset", tuple)
 
-    want_main_area = True
-    resizing_enabled = True
-    compress_signal = False
+    want_main_area = False
 
     _dims = Setting(dict(), schema_only=True)
 
     def __init__(self):
         super().__init__()
-        layout = gui.vBox(self.mainArea, 'dimensions').layout()
         self._widget = DimensionWidget(parent=self)
-        layout.addWidget(self._widget)
+        self.controlArea.layout().addWidget(self._widget)
 
         # buttons
         types = qt.QDialogButtonBox.Ok
         self.buttons = qt.QDialogButtonBox(parent=self)
         self.buttons.setStandardButtons(types)
-        layout.addWidget(self.buttons)
+        self.controlArea.layout().addWidget(self.buttons)
 
         self.buttons.accepted.connect(self.validate)
         self.buttons.button(qt.QDialogButtonBox.Ok).setEnabled(False)
@@ -101,29 +101,38 @@ class DimensionWidgetOW(OWWidget):
         return self._widget.dataset
 
     @Inputs.dataset
-    def setDataset(self, dataset):
+    def setDataset(self, _input):
         """
         Input signal to set the dataset.
         """
-        self.buttons.button(qt.QDialogButtonBox.Ok).setEnabled(False)
-        if dataset is not None:
+        try:
+            dataset, update = _input
+            self.buttons.button(qt.QDialogButtonBox.Ok).setEnabled(False)
             try:
                 self._widget.setDataset(*dataset)
                 # load properties
-                _dims = self._convertDictToDim(self._dims)
+                _dims = utils.convertDictToDim(self._dims)
                 self._widget.setDims(_dims)
             except ValueError as e:
                 qt.QMessageBox.warning(self,
                                        'Fail to setup dimension definition',
                                        str(e))
             else:
-                self.show()
+                if update is None:
+                    self.open()
+                elif update != self:
+                    self.Outputs.dataset.send(((self,) + self._widget.getDataset(), update))
+        except Exception:
+            self.Outputs.dataset.send(None)
+
+    def _updateDataset(self, widget, dataset):
+        self._widget._updateDataset(widget, dataset)
 
     def validate(self):
         """
         Tries to fit the dimensions into the dataset.
         """
-        self.Outputs.dataset.send(self._widget.getDataset())
+        self.Outputs.dataset.send(((self,) + self._widget.getDataset(), None))
         self.updateProperties()
         OWWidget.accept(self)
 
@@ -139,17 +148,3 @@ class DimensionWidgetOW(OWWidget):
                 assert isinstance(_dim, Dimension)
                 axis = _dim.axis
                 self._dims[axis] = _dim.to_dict()
-
-    def _convertDictToDim(self, _dict):
-        """
-        Convert dictionary of dimensions from Settings to Dimension.
-        """
-        if len(_dict) > 0:
-            _dims = {}
-            for _axis, _dim in _dict.items():
-                assert type(_dim) is dict
-                _dims[_axis] = Dimension.from_dict(_dim)
-            return _dims
-
-        else:
-            return {}
