@@ -26,13 +26,12 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "14/08/2019"
+__date__ = "08/04/2020"
 
 
 from Orange.widgets.widget import OWWidget, Input, Output
-
-from darfix.core.dataset import Dataset
 from darfix.gui.metadataWidget import MetadataWidget
+from darfix.core.process import DataPassThrough
 
 
 class MetadataWidgetOW(OWWidget):
@@ -43,14 +42,15 @@ class MetadataWidgetOW(OWWidget):
     name = "metadata"
     icon = "icons/metadata.svg"
     want_main_area = False
+    ewokstaskclass = DataPassThrough
 
     # Inputs
     class Inputs:
-        dataset = Input("dataset", Dataset)
+        dataset = Input("dataset", tuple)
 
     # Outputs
     class Outputs:
-        dataset = Output("dataset", Dataset)
+        dataset = Output("dataset", tuple)
 
     def __init__(self):
         super().__init__()
@@ -59,8 +59,10 @@ class MetadataWidgetOW(OWWidget):
         self.controlArea.layout().addWidget(self._widget)
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
-            self._widget.setDataset(dataset)
-        else:
-            self._widget.clearTable()
+    def setDataset(self, _input):
+        if _input:
+            dataset, update = _input
+            if dataset:
+                self._widget.setDataset(*dataset)
+            else:
+                self._widget.clearTable()
