@@ -32,6 +32,7 @@ from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
 from silx.gui.colors import Colormap
 from darfix.gui.noiseRemovalWidget import NoiseRemovalDialog
+from darfix.core.process import NoiseRemoval
 
 
 class NoiseRemovalWidgetOW(OWWidget):
@@ -42,6 +43,7 @@ class NoiseRemovalWidgetOW(OWWidget):
     name = "noise removal"
     icon = "icons/noise_removal.png"
     want_main_area = False
+    ewokstaskclass = NoiseRemoval
 
     # Inputs
     class Inputs:
@@ -69,9 +71,9 @@ class NoiseRemovalWidgetOW(OWWidget):
         self._widget.okSignal.connect(self._sendSignal)
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        self.show()
-        if dataset:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
             self._widget.setDataset(*dataset)
             if self.method:
                 self._widget.mainWindow.method = self.method
@@ -85,15 +87,23 @@ class NoiseRemovalWidgetOW(OWWidget):
                 self._widget.mainWindow.step = self.step
             if self.chunks:
                 self._widget.mainWindow.chunks = self.chunks
-        else:
+
+            if update is None:
+                self.show()
+            elif update != self:
+                self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), update))
+        except Exception:
             self._widget.mainWindow.clearStack()
             # Emit None
-            self.Outputs.dataset.send(dataset)
+            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
         self._widget.mainWindow.setStackViewColormap(colormap)
 
+    def _updateDataset(self, widget, dataset):
+        self._widget.mainWindow._updateDataset(widget, dataset)
+
     def _sendSignal(self):
         """
         Function to emit the new dataset.
@@ -105,6 +115,6 @@ class NoiseRemovalWidgetOW(OWWidget):
         self.step = self._widget.mainWindow.step
         self.chunks = self._widget.mainWindow.chunks
         self.bottom_threshold = self._widget.mainWindow.bottom_threshold
-        self.Outputs.dataset.send(self._widget.mainWindow.getDataset())
+        self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), None))
         self.Outputs.colormap.send(self._widget.mainWindow.getStackViewColormap())
         self.close()
