@@ -33,6 +33,7 @@ from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input
 from silx.gui.colors import Colormap
 from darfix.gui.rockingCurvesWidget import RockingCurvesWidget
+from darfix.core.process import RockingCurves
 
 
 class RockingCurvesWidgetOW(OWWidget):
@@ -43,6 +44,7 @@ class RockingCurvesWidgetOW(OWWidget):
     name = "rocking curves"
     icon = "icons/curves.png"
     want_main_area = False
+    ewokstaskclass = RockingCurves
 
     # Inputs
     class Inputs:
@@ -62,10 +64,14 @@ class RockingCurvesWidgetOW(OWWidget):
             self._widget.intThresh = self.int_thresh
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
             self._widget.setDataset(*dataset)
-            self.open()
+            if update is None:
+                self.open()
+        except Exception:
+            pass
 
     @Inputs.colormap
     def setColormap(self, colormap):
