@@ -26,17 +26,17 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "05/12/2019"
+__date__ = "11/12/2020"
 
 
 # import numpy
 
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
+from silx.gui.colors import Colormap
 # from silx.gui import qt
 from darfix.gui.roiSelectionWidget import ROISelectionWidget
-from darfix.core.dataset import Dataset
-# from darfix.core.roi import apply_3D_ROI
+from darfix.core.process import RoiSelection
 
 
 class RoiSelectionWidgetOW(OWWidget):
@@ -44,13 +44,16 @@ class RoiSelectionWidgetOW(OWWidget):
     name = "roi selection"
     icon = "icons/roi.png"
     want_main_area = False
+    ewokstaskclass = RoiSelection
 
     # Inputs/Outputs
     class Inputs:
-        dataset = Input("dataset", Dataset)
+        dataset = Input("dataset", tuple)
+        colormap = Input("colormap", Colormap)
 
     class Outputs:
-        dataset = Output("dataset", Dataset)
+        dataset = Output("dataset", tuple)
+        colormap = Output("colormap", Colormap)
 
     # Settings
     roi_origin = Setting(list(), schema_only=True)
@@ -69,19 +72,29 @@ class RoiSelectionWidgetOW(OWWidget):
         # self._button.pressed.connect(self._createRoi)
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
-            self._widget.setDataset(dataset=dataset)
-        else:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
+            self._widget.setDataset(*dataset)
+            # Set saved roi
+            if len(self.roi_origin) and len(self.roi_size):
+                self._widget.setRoi(origin=self.roi_origin, size=self.roi_size)
+
+            if update is None:
+                self.open()
+            elif update != self:
+                self.Outputs.dataset.send(((self,) + self._widget.getDataset(), update))
+        except Exception:
             self._widget.clearStack()
             # Emit None
-            self.Outputs.dataset.send(dataset)
+            self.Outputs.dataset.send(None)
 
-        # Set saved roi
-        if len(self.roi_origin) and len(self.roi_size):
-            self._widget.setRoi(origin=self.roi_origin, size=self.roi_size)
+    @Inputs.colormap
+    def setColormap(self, colormap):
+        self._widget.setStackViewColormap(colormap)
 
-        self.open()
+    def _updateDataset(self, widget, dataset):
+        self._widget._updateDataset(widget, dataset)
 
     def _sendSignal(self, roi_origin=[], roi_size=[]):
         """
@@ -90,4 +103,5 @@ class RoiSelectionWidgetOW(OWWidget):
         self.close()
         self.roi_origin = roi_origin
         self.roi_size = roi_size
-        self.Outputs.dataset.send(self._widget.dataset)
+        self.Outputs.dataset.send(((self,) + self._widget.getDataset(), None))
+        self.Outputs.colormap.send(self._widget.getStackViewColormap())
